@@ -17,7 +17,7 @@
 import os
 import struct
 from io import BytesIO, BufferedIOBase
-from typing import Dict, List, Optional, BinaryIO, Any, Tuple, Union
+from typing import Dict, List, Optional, BinaryIO, Tuple, Union
 
 # Third party imports
 import numpy as np
@@ -25,7 +25,8 @@ from nptyping import Array
 from PIL import Image
 
 # Self imports
-from flyr.thermogram import FlyrThermogram
+from .thermogram import FlyrThermogram
+from .palette_info import Palette
 
 
 # Constants
@@ -67,7 +68,9 @@ def unpack(path_or_stream: Union[str, BinaryIO]) -> FlyrThermogram:
     """
     if isinstance(path_or_stream, str) and os.path.isfile(path_or_stream):
         with open(path_or_stream, "rb") as flirh:
-            return unpack(flirh)
+            thermogram = unpack(flirh)
+        thermogram.path = path_or_stream
+        return thermogram
     elif isinstance(path_or_stream, BufferedIOBase):
         stream = path_or_stream
         flir_app1_stream = extract_flir_app1(stream)
@@ -199,6 +202,7 @@ def parse_thermal(
     RECORD_IDX_RAW_DATA = 1
     RECORD_IDX_EMBEDDED_IMAGE = 14
     RECORD_IDX_CAMERA_INFO = 32
+    RECORD_IDX_PALETTE_INFO = 34
 
     raw_data_md = records[RECORD_IDX_RAW_DATA]
     width, height, raw_data = parse_raw_data(stream, raw_data_md)
@@ -213,7 +217,15 @@ def parse_thermal(
     camera_info_md = records[RECORD_IDX_CAMERA_INFO]
     camera_info = parse_camera_info(stream, camera_info_md)
 
-    thermogram = FlyrThermogram(raw_data, camera_info, optical=embedded_image)
+    try:
+        palette_info_md = records[RECORD_IDX_PALETTE_INFO]
+        palette_info: Optional[Palette] = parse_palette_info(stream, palette_info_md)
+    except:
+        palette_info = None
+
+    thermogram = FlyrThermogram(
+        raw_data, camera_info, optical=embedded_image, palette=palette_info
+    )
     return thermogram
 
 
@@ -298,7 +310,7 @@ def parse_flir_record_metadata(
 def parse_raw_data(
     stream: BinaryIO, metadata: Tuple[int, int, int, int]
 ) -> Tuple[int, int, Array[np.uint8, ..., ...]]:
-    (entry_idx, _, offset, length) = metadata
+    (_, _, offset, length) = metadata
     stream.seek(offset)
 
     # from PIL import ImageFile  # Uncomment to enable loading corrupt PNGs
@@ -308,16 +320,27 @@ def parse_raw_data(
     width = int.from_bytes(stream.read(2), "little")
     height = int.from_bytes(stream.read(2), "little")
 
-    stream.seek(offset + 2 * 16)  # TODO document why 2 * 16
-    # data_format = stream.read(4)  # TODO Use format to support different FLIRs
-    # print(width, height, type)
-    # stream.seek(offset + 2*16)
-
     # Read the bytes with the raw thermal data and decode using PIL
+    length = min(width * height * 2, length)
+    stream.seek(offset + 2 * 16)  # TODO document why 2 * 16
     thermal_bytes = stream.read(length)
     thermal_stream = BytesIO(thermal_bytes)
-    thermal_img = Image.open(thermal_stream)
-    thermal_np = np.array(thermal_img)
+
+    # Some thermograms have the raw data stored as PNG files, others simply the
+    # sequence of bytes
+    if thermal_bytes[:4] != b"\x89PNG":
+        u16s = []
+        u16 = thermal_stream.read(2)
+        while u16:
+            u16s.append(int.from_bytes(u16, "little"))
+            u16 = thermal_stream.read(2)
+        thermal_np = np.array(u16s).reshape((height, width))
+    else:
+        # FLIR PNG data is in the wrong byte order
+        thermal_img = Image.open(thermal_stream)
+        thermal_np = np.array(thermal_img)
+        fix_byte_order = np.vectorize(lambda x: (x >> 8) + ((x & 0x00FF) << 8))
+        thermal_np = fix_byte_order(thermal_np)
 
     # Check shape
     if thermal_np.shape != (height, width):
@@ -325,10 +348,6 @@ def parse_raw_data(
         msg = msg.format(thermal_np.shape, height, width)
         raise ValueError(msg)
 
-    # FLIR PNG data is in the wrong byte order, fix that
-    fix_byte_order = np.vectorize(lambda x: (x >> 8) + ((x & 0x00FF) << 8))
-    thermal_np = fix_byte_order(thermal_np)
-
     return width, height, thermal_np
 
 
@@ -365,8 +384,7 @@ def parse_embedded_image(
 def parse_camera_info(
     stream: BinaryIO, metadata: Tuple[int, int, int, int]
 ) -> Dict[str, Union[int, float]]:
-    (entry_idx, _, offset, length) = metadata
-    stream.seek(offset + 32)
+    (_, _, offset, _) = metadata
     stream.seek(offset + 32)
 
     emissivity = stream.read(4)
@@ -395,6 +413,14 @@ def parse_camera_info(
     planck_o = stream.read(4)
     planck_r2 = stream.read(4)
 
+    stream.seek(offset + 784)
+    raw_value_range_min = stream.read(2)
+    raw_value_range_max = stream.read(2)
+
+    stream.seek(offset + 824)
+    raw_value_median = stream.read(4)
+    raw_value_range = stream.read(4)
+
     # Create result dictionary
     float_from_bytes = lambda v: struct.unpack("f", v)[0]
     camera_info = {
@@ -415,6 +441,10 @@ def parse_camera_info(
         "atmospheric_trans_beta1": float_from_bytes(atmospheric_trans_beta1),
         "atmospheric_trans_beta2": float_from_bytes(atmospheric_trans_beta2),
         "atmospheric_trans_x": float_from_bytes(atmospheric_trans_x),
+        "raw_value_range_min": struct.unpack("H", raw_value_range_min)[0],
+        "raw_value_range_max": struct.unpack("H", raw_value_range_max)[0],
+        "raw_value_median": struct.unpack("i", raw_value_median)[0],
+        "raw_value_range": struct.unpack("i", raw_value_range)[0],
     }
 
     to_round = [
@@ -426,18 +456,92 @@ def parse_camera_info(
         k: round(v, 2) if k in to_round else v for k, v in camera_info.items()
     }
 
-    # Filter out zero values
-    return {k: v for k, v in camera_info.items() if v}
+    return camera_info
+
+
+def parse_palette_info(
+    stream: BinaryIO, metadata: Tuple[int, int, int, int]
+) -> Palette:
+    (_, _, offset, _) = metadata
+
+    stream.seek(offset)
+    num_colors = int.from_bytes(stream.read(6), "little")
+
+    def read_color_tuple(stream):
+        c1 = int.from_bytes(stream.read(1), "little")
+        c2 = int.from_bytes(stream.read(1), "little")
+        c3 = int.from_bytes(stream.read(1), "little")
+        return (c1, c2, c3)
+
+    above = read_color_tuple(stream)
+    below = read_color_tuple(stream)
+    overflow = read_color_tuple(stream)
+    underflow = read_color_tuple(stream)
+    isotherm1 = read_color_tuple(stream)
+    isotherm2 = read_color_tuple(stream)
+    stream.read(2)
+    method = int.from_bytes(stream.read(1), "little")
+    stretch = int.from_bytes(stream.read(1), "little")
+
+    stream.seek(offset + 112)
+    palette: bytes = stream.read(num_colors * 3)
+    palette_ycc: List[Tuple[int, int, int]] = [
+        (palette[i], palette[i + 1], palette[i + 2]) for i in range(0, len(palette), 3)
+    ]
+
+    return Palette(
+        num_colors,
+        above,
+        below,
+        overflow,
+        underflow,
+        isotherm1,
+        isotherm2,
+        method,
+        stretch,
+        "",  # TODO Read palette file name
+        "",  # TODO Read palette name
+        palette_ycc,
+    )
+
+
+def main():
+    import argparse
+    import warnings
+    import traceback as tb
+    from pprint import pprint
+
+    parser = argparse.ArgumentParser()
+    parser.add_argument(
+        "--camera-info", "-c", action="store_true", default=False, required=False
+    )
+    parser.add_argument("--palette-dest", "-p", default="", required=False)
+    parser.add_argument("--render-dest", "-r", default="", required=False)
+    parser.add_argument("thermogram")
+    args = parser.parse_args()
+
+    script = os.path.basename(__file__)
+    if not os.path.isfile(args.thermogram):
+        print(f"{script}: error: file {args.thermogram} doesn't exist")
+        exit(1)
+
+    try:
+        thermogram = unpack(args.thermogram)
+
+        if args.camera_info:
+            pprint(thermogram.metadata)
+
+        if args.palette_dest and thermogram.palette is not None:
+            thermogram.palette.render_pil().save(args.palette_dest)
+        elif args.palette_dest:
+            warnings.warn(f"{script}: warning: no palette was extracted")
+
+        if args.render_dest:
+            thermogram.render_pil().save(args.render_dest)
+    except Exception as e:
+        msg = f"{tb.format_exc()}\n{script}: error: failed unpacking {args.thermogram}"
+        print(msg)
 
 
 if __name__ == "__main__":
-    import sys
-
-    if not len(sys.argv) == 2:
-        print("Usage: flyr.py <path_to_flir_file>")
-        exit()
-
-    # TODO Do something useful when called as script
-    file_path = sys.argv[1]
-    celsius = unpack(file_path).celsius
-    print(celsius)
+    main()
