@@ -32,9 +32,10 @@ class FlyrThermogram:
     """
 
     # Required members variables required to be set
-    __thermal = None  # type: Array[int, ..., ..., ...]
-    __metadata = None  # type: Dict[str, Union[float, int]]
-    __optical = None  # type: Optional[Array[int, ..., ..., 3]]
+    __thermal: Array[np.int64, ..., ...]
+    __metadata: Dict[str, Union[float, int]]
+    __metadata_adjustments: Dict[str, Union[float, int]]
+    __optical: Optional[Array[int, ..., ..., 3]]
     palette: Optional[pinfo.Palette]
     path: Optional[str]
 
@@ -45,6 +46,7 @@ class FlyrThermogram:
         optical: Optional[Array[np.uint8, ..., ..., 3]] = None,
         palette: Optional[pinfo.Palette] = None,
         path: Optional[str] = None,
+        metadata_adjustments: Dict[str, Union[float, int]] = {},
     ):
         """ Initialize a new instance of this class. The raw thermal data and
             the accompanying metadata to correctly interpret this data are
@@ -55,13 +57,25 @@ class FlyrThermogram:
             thermal: Array[np.int64, ..., ...]
                 A 2D numpy array of 64 bit integers. This is the raw thermal
                 data as it is stored in the FLIR file. Order is [H, W].
-            metadata: Dict[str, Any]
+            metadata: Dict[str, Union[float, int]]
                 A dictionary with physical parameters to interpret the raw
-                thermal data.
+                thermal data, as it was included in the original FLIR file.
             optical: Array[np.uint8, ..., ..., 3]
                 A 3D numpy array of 8 bit integers, in the order of [H, W, C].
                 This should be a 'normal' photo (RGB) of the same scene as
                 thermogram.
+            palette: Optional[pinfo.Palette]
+                The palette embedded in the file, which can be used to render
+                with by default. Is `None` by default in which case a grayscale
+                palette is used.
+            path: Optional[str]
+                The path to the original file. Is only used to determine this
+                this object's identifier and can safely be left `None`
+                (default).
+            metadata_adjustments: Dict[str, Union[float, int]]
+                A dictionary with adjustments to the above dictionary. This
+                separation allows both using the original settings and
+                calculating temperatures with different physical parameters.
 
             Returns
             -------
@@ -69,7 +83,8 @@ class FlyrThermogram:
         """
         self.__thermal = thermal  # Raw thermal data
         self.__optical = optical  # Optical (RGB) photo in the thermogram
-        self.__metadata = metadata
+        self.__metadata = metadata.copy()
+        self.__metadata_adjustments = metadata_adjustments.copy()
         self.palette = palette
         self.path = path
 
@@ -135,7 +150,9 @@ class FlyrThermogram:
 
     @property
     def metadata(self) -> Dict[str, Union[float, int]]:
-        return self.__metadata.copy()
+        metadata = self.__metadata.copy()
+        metadata.update(self.__metadata_adjustments)
+        return metadata
 
     @property
     def identifier(self) -> Optional[str]:
@@ -291,21 +308,27 @@ class FlyrThermogram:
         msg = f"Parameter in_place incorrectly not of type bool but {type(in_place)}. Be sure to pass it first."
         assert isinstance(in_place, bool), msg
         if in_place:
-            self.__metadata.update(kwargs)
+            self.__metadata_adjustments.update(kwargs)
             return self
 
-        thermal = self.__thermal.copy()
-        optical = None if self.__optical is None else self.__optical.copy()
-        metadata = self.metadata
-        metadata.update(kwargs)
-        return FlyrThermogram(thermal, metadata, optical)
+        metadata_adjustments = self.__metadata_adjustments.copy()
+        metadata_adjustments.update(kwargs)
+
+        return FlyrThermogram(
+            self.__thermal.copy(),
+            self.__metadata.copy(),
+            None if self.__optical is None else self.__optical.copy(),
+            palette=self.palette,
+            metadata_adjustments=metadata_adjustments,
+            path=self.path,
+        )
 
     def embedded_range(self, unit: str) -> Tuple[float, float]:
         range_median = self.__metadata["raw_value_median"]
         range_half = self.__metadata["raw_value_range"] / 2
 
         raw_vals = np.array([[range_median - range_half, range_median + range_half]])
-        min_v, max_v = self.__raw_to_kelvin_with_metadata(raw_vals).squeeze()
+        min_v, max_v = self.__raw_to_kelvin_with_metadata(raw_vals, orig=True).squeeze()
         if unit == "celsius":
             min_v = min_v - 273.15
             max_v = max_v - 273.15
@@ -315,26 +338,32 @@ class FlyrThermogram:
 
         return (min_v, max_v)
 
-    def __raw_to_kelvin_with_metadata(self, thermal) -> Array[np.float64, ..., ...]:
+    def __raw_to_kelvin_with_metadata(
+        self, thermal, orig: bool = False
+    ) -> Array[np.float64, ..., ...]:
+        metadata = self.__metadata.copy()
+        if not orig:
+            metadata.update(self.__metadata_adjustments)
+
         return FlyrThermogram.__raw_to_kelvin(
             thermal,
-            self.__metadata["emissivity"],
-            self.__metadata["object_distance"],
-            self.__metadata["atmospheric_temperature"],
-            self.__metadata["ir_window_temperature"],
-            self.__metadata["ir_window_transmission"],
-            self.__metadata["reflected_apparent_temperature"],
-            self.__metadata["relative_humidity"],
-            self.__metadata["planck_r1"],
-            self.__metadata["planck_r2"],
-            self.__metadata["planck_b"],
-            self.__metadata["planck_f"],
-            self.__metadata["planck_o"],
-            self.__metadata["atmospheric_trans_alpha1"],
-            self.__metadata["atmospheric_trans_alpha2"],
-            self.__metadata["atmospheric_trans_beta1"],
-            self.__metadata["atmospheric_trans_beta2"],
-            self.__metadata["atmospheric_trans_x"],
+            metadata["emissivity"],
+            metadata["object_distance"],
+            metadata["atmospheric_temperature"],
+            metadata["ir_window_temperature"],
+            metadata["ir_window_transmission"],
+            metadata["reflected_apparent_temperature"],
+            metadata["relative_humidity"],
+            metadata["planck_r1"],
+            metadata["planck_r2"],
+            metadata["planck_b"],
+            metadata["planck_f"],
+            metadata["planck_o"],
+            metadata["atmospheric_trans_alpha1"],
+            metadata["atmospheric_trans_alpha2"],
+            metadata["atmospheric_trans_beta1"],
+            metadata["atmospheric_trans_beta2"],
+            metadata["atmospheric_trans_x"],
         )
 
     @staticmethod
